package com.jiuqi.script.rhino;

import org.mozilla.javascript.Context;
import org.mozilla.javascript.Callable;
import java.lang.reflect.Method;
import org.mozilla.javascript.Scriptable;
import java.util.Map;
import org.mozilla.javascript.ScriptableObject;

public class LimitedScriptObject extends ScriptableObject
{
    private static final long serialVersionUID = 8048574707866032033L;
    private static final Object[] emptyArray;
    private Object self;
    private Map methods;
    
    static {
        emptyArray = new Object[0];
    }
    
    public LimitedScriptObject(final Object obj, final Map methods) {
        this.self = obj;
        this.methods = methods;
    }
    
    public String getClassName() {
        return "LimitedScriptObject";
    }
    
    public Object[] getAllIds() {
        return this.getIds();
    }
    
    public Object[] getIds() {
        return this.methods.keySet().toArray();
    }
    
    public Object get(final String name, final Scriptable start) {
        final Method m = this.methods.get(name);
        if (m != null) {
            return new Callable() {
                public Object call(final Context context, final Scriptable scope, final Scriptable thisObj, final Object[] args) {
                    try {
                        if (args == null) {
                            return m.invoke(LimitedScriptObject.this.self, LimitedScriptObject.emptyArray);
                        }
                        final Object[] newArgs = new Object[args.length];
                        for (int i = 0; i < args.length; ++i) {
                            newArgs[i] = ScriptHelper._toJava(args[i], scope);
                        }
                        return m.invoke(LimitedScriptObject.this.self, newArgs);
                    }
                    catch (final Throwable e) {
                        e.printStackTrace();
                        throw new RuntimeException(e);
                    }
                }
            };
        }
        String firstChar = name.substring(0, 1);
        firstChar = firstChar.toUpperCase();
        final String getter = "get" + firstChar + name.substring(1);
        final Method m2 = this.methods.get(getter);
        if (m2 == null) {
            throw new IllegalArgumentException("Unknown method or property:" + name);
        }
        Object result = null;
        try {
            result = m2.invoke(this.self, LimitedScriptObject.emptyArray);
        }
        catch (final Exception e) {
            e.printStackTrace();
        }
        return result;
    }
    
    public boolean has(final String name, final Scriptable start) {
        return this.methods.containsKey(name);
    }
}
